::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::
:: Description:     Cleans up after an installation of TiVo Desktop 2.3 or later.
::                  
::
:: Local Notices:   Copyright (c) 2009 TiVo Inc, all rights reserved.
::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

@echo off

cd %~dp0
 

::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:Main

    setlocal EnableExtensions EnableDelayedExpansion

	:: Clean the screen

        cls

        :: Greet the user, let them know what is about to happen.

        echo.
        echo -------------------------------------------------------------------------------
        echo.
        echo TivoDesktopCleaner, version 2.8.1
        echo.
        echo This utility completely removes Tivo Desktop versions 2.3 - 2.8.1
        echo from your computer. For best results, close any other programs
        echo that may be running and avoid interacting with your computer
        echo until this process is finished. Don't worry, it should not take long. 
        echo.
        echo -------------------------------------------------------------------------------
        echo.


        :: Do they really want to go through with it?

        call :PromptUser "Do you want to continue? (Y/N) "

        if /i "%RETVAL%" NEQ "Y" goto :exit_Main

        :: Fire things up, make sure we can write temporary files, and then go ahead with it!

        call :Initialize

        :: Check for cleaning all registry keys (including Plus and MAK)

        if "%1"=="/all" (
          set CleanAll=YES
        )

        :: Let the user write Plus key down and keep it safely. 

        if "%CleanAll%"=="YES" (
          REM
          pushd "%~dp0"
          cscript //Nologo GetPlusUpgradeKey.js
          popd
          REM
          PAUSE
        )


        mkdir %TempPath% > nul 2>&1

        call :zapdesktop


        :: Let the user know how things went.
        echo -------------------------------------------------------------------------------
        echo.
        echo TiVo Desktop has been removed from your system.
        echo.
        echo -------------------------------------------------------------------------------
        echo.
        pause

:exit_Main

    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:Initialize


        :: Grab some per-installation values from the registry.  If they cannot be found, we'll
        :: default to something "safe" in order to allow the script to execute reasonably.

        call :QueryRegistry "HKCU\Software\TiVo\Desktop" InstallPath MISSING

        if "%RETVAL%" NEQ "MISSING" (

            set WasInstalled=YES

        ) else (

            set RETVAL=C:\Program Files\TiVo\Desktop\
            set WasInstalled=NO
        )

        set InstallPath=%RETVAL:~0,-1%

        call :QueryRegistry "HKCU\Software\TiVo\Desktop\Modules\Config\TivoNowPlaying" LocalRootPath "%UserProfile%\My Documents\My TiVo Recordings"

        set LocalRootPath=%RETVAL%


        :: Okay, collect and assemble some other "globals" that we'll use throughout the script.

        set MsiProductCode={4E839090-3B68-436A-B3CF-A2A08C38DD26}
        set PlusPath=%InstallPath%\Plus
        set OldPlusPath=%InstallPath%\Support
        set SharedPath=%CommonProgramFiles(x86)%\TiVo Shared
	
        if "%SharedPath%" EQU "\TiVo Shared" (
            set SharedPath=%CommonProgramFiles%\TiVo Shared
        )
         
        set BeaconPath=%SharedPath%\Beacon
        set TransferPath=%SharedPath%\Transfer
        set DirectShowPath=%SharedPath%\DirectShow
        set DataPath=%UserProfile%\Local Settings\Application Data\TiVo Desktop
        set TempPath=%TEMP%\TiVoDesktopCleaner
        set LogFile=%TempPath%\Log.txt

        ::  Set up some "globals" that we'll use for the cleaning Plus key from the registry.
        
        set CleanAll="NO"
        set TivoCurrentUserKey=HKCU\Software\TiVo
        set PlusUpgradeLicenseKey=HKCU\Software\TiVo\Desktop
        set PlusUpgradeLicenseKey23=HKLM\Software\TiVo\Desktop
        set SharingKeys=HKCU\Software\TiVo\SharingKeys
        set DecoderKeys="HKCU\Software\MainConcept (Consumer)\MainConcept (Consumer) AVC/H.264 Video Decoder\TranscoderHost.exe"

    goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:zapdesktop

    setlocal


        :: While we log, perform all of our removal steps.

        call :OpenLog

            echo.
            echo Stopping software...

                call :StopSoftware
                call :KillSoftware

                call :RemoveSoftware
                call :KillSoftware

            echo Done.

            echo.
            echo Cleaning registry...

                call :UnregisterComponents
                call :CleanupRegistry

            echo Done.

            echo.
            echo Zapping installation...

                call :ZapInstallation

            echo Done.

            echo.
            echo Deleting files...

                call :CleanupFiles

            echo Done.

        call :CloseLog


    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:ZapInstallation

    setlocal DisableDelayedExpansion


        :: We use the Microsoft-provided tool to remove all traces the most recent TiVo Desktop
        :: installation from the registry (which, among other things, should certainly make the
        :: Add/Remove Programs entry for it disappear).
        ::
        :: T = remove all info for given product code
        :: W = for all users (by default, only for the current user)
        :: G = remove orphaned cached Windows Installer data files (for all users)
        :: ! = force 'yes' response to any prompt
        pushd "%~dp0"
        call :ExecuteLoggedCommand MsiZap TWG! %MsiProductCode%
        popd

    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:StopSoftware

    setlocal


        :: Try to exit the user-interfaces (sends polite WM_CLOSE).

        call :CloseWindow TivoDesktop.*
        call :CloseWindow LogFinder.*
        call :CloseWindow WmPlayer.*
        call :CloseWindow TiVoDiag.*


        :: Politely ask all the services to stop.

        call :ServiceCommand "%InstallPath%\TivoServer" /stop
        call :ServiceCommand "%InstallPath%\TivoNotify" /stop
        call :ServiceCommand "%TransferPath%\TivoTransfer" /stop
        call :ServiceCommand "%BeaconPath%\TivoBeacon" /stop


        :: Use the Service Control Manager to stop the TiVo Beacon (in case /stop didn't work).

        call :ServiceControl TivoBeacon2 stop


    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:KillSoftware

    setlocal


        :: Forcibly terminate any software that is still running.

        call :Terminate TivoDesktop.*
        call :Terminate LogFinder.*
        call :Terminate TivoConverter.*
        call :Terminate TranscodingService.*
        call :Terminate TivoWmlPublisher.*
        call :Terminate TivoServer.*
        call :Terminate TivoNotify.*
        call :Terminate TivoTransfer.*
        call :Terminate TivoBeacon.*
        call :Terminate WmPlayer.*
        call :Terminate TiVoDiag.*


    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:RemoveSoftware

    setlocal


        :: Ask all the services to remove themselves.

        call :ServiceCommand "%InstallPath%\TivoServer" /remove
        call :ServiceCommand "%InstallPath%\TivoNotify" /remove /unregister
        call :ServiceCommand "%TransferPath%\TivoTransfer" /unregserver
        call :ServiceCommand "%BeaconPath%\TivoBeacon" /remove


        :: Use the Service Control Manager to stop the TiVo Beacon (in case /remove didn't work).

        call :ServiceControl TivoBeacon2 delete


    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:UnregisterComponents

    setlocal


        :: Locate all the files in the Plus folder, passing each to UnregisterServer.

        for %%I in ("%PlusPath%\*.*") do (

            call :UnregisterServer "%%~fI"
        )


        :: Do the same for the "old" Plus folder, called Support.  (This was the name in TiVo
        :: Desktop 2.3 and 2.3a; it changed to Plus in 2.4.)

        for %%I in ("%OldPlusPath%\*.*") do (

            call :UnregisterServer "%%~fI"
        )


        :: Specifically unregister our custom DirectShow source filter.

        call :UnregisterServer "%DirectShowPath%\TivoDirectShowFilter.dll"


    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:CleanupFiles

    setlocal

        :: Read all the non-comment and non-blank line(s) from the configuration file and treat
        :: each as the set of parameter(s) for calling DeleteFileOrFolder.

        pushd "%~dp0"
        for /f "tokens=*" %%I in (Files.lst) do (

            call :DeleteFileOrFolder %%I
        )
        popd


    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:CleanupRegistry

    setlocal


        :: Read all the non-comment and non-blank line(s) from the configuration file and treat
        :: each as the set of parameter(s) for calling DeleteKeyOrValue.

        pushd "%~dp0"
        for /f "tokens=*" %%I in (Registry.lst) do (

            call :DeleteKeyOrValue %%I
        )
        popd

        call :DeleteKeyOrValue %PlusUpgradeLicenseKey% FirstTimeLaunchWizard

        :: Clean Plus and MAK keys from the registry if it is necessary.
        
        if "%CleanAll%"=="YES" (
            call :DeleteKeyOrValue %PlusUpgradeLicenseKey%
            call :DeleteKeyOrValue %PlusUpgradeLicenseKey23%
            call :DeleteKeyOrValue %SharingKeys%
            call :DeleteKeyOrValue %DecoderKeys%
            call :DeleteKeyOrValue %TivoCurrentUserKey%
        )

    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::
:: %1 - Path of the key to query.
:: %2 - Name of the value to query.
:: %3 - Default data (optional).
::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:QueryRegistry

    setlocal


        :: Execute the registry query command, parsing the "found" information.  Parsing consists
        :: of skipping to the fifth line, placing the first two whitespace-delimited tokens (name
        :: and type) into variables I and J, and the remainder of the line (the value, containing
        :: additional white space of its own perhaps) into variable K.

        set Data=
        
        for /f "tokens=1,2,*" %%I in ('reg query %1 /v %2 2^> nul') do (

            set Data=%%K
        )


        :: If we have no data at this point, then we'll just use the (possibly empty) default.

        if "%Data%" EQU "" (

            set Data=%~3
        )


    endlocal & set "RETVAL=%Data%" & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::
:: %1 - Path of the service exectuable (TivoServer.exe, etc.).
:: %2 - Service command switch (see logic in Calypso\Win\Src\Common\ProcessServiceCommand.cpp and
::      Calypso\Win\Notify\ClassManager.cpp for switch details).
::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:ServiceCommand

    setlocal

        call :ExecuteLoggedCommand %1 %2 %3 %4 %5

    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::
:: %1 - Name of the service.
:: %2 - Service command name (syntax is identical to the "command" parameter of the SC.exe system
::      utility).
::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:ServiceControl

    setlocal

        call :ExecuteLoggedCommand sc %2 %1

    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::
:: %1 - Name of the process image to close (syntax is identical to the "imagename" parameter of
::      the TaskKill.exe system utility).
::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:CloseWindow

    setlocal
        pushd "%~dp0"
        call :ExecuteLoggedCommand taskkill /im %1
        popd 

    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::
:: %1 - Name of the process image to terminate (syntax is identical to the "imagename" parameter
::      of the TaskKill.exe system utility).
::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:Terminate

    setlocal
        pushd "%~dp0"
        call :ExecuteLoggedCommand taskkill /f /im %1
        popd        
 
    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::
:: %1 - Path of the server executable to unregister (.dll, .ax, etc.).
::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:UnregisterServer

    setlocal
		
        call :ExecuteLoggedCommand regsvr32 /s /u %1

    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::
:: %1 - Path of the file or folder to check.
::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:IsFolder

    setlocal


        :: If we can "cd into" the provided path, then we assume it must be a folder.  Note that
        :: we make sure to "cd out" again if this was indeed the case, so we don't mess up the
        :: current directory.

        pushd %1 > nul 2>&1

        if ERRORLEVEL 1 (

            set IsFolder=NO

        ) else (

            popd & set IsFolder=YES
        )


    endlocal & set "RETVAL=%IsFolder%" & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::
:: %1 - Path of the file or folder to delete.
::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:DeleteFileOrFolder

    setlocal


        :: Passed a file or a folder?

        call :IsFolder %1


        :: For a folder, first we clear the attributes on everything within (to avoid trouble with
        :: the rmdir), as well as the folder itself, before finally deleting the folder.  For a
        :: file, we simply clear its attributes (to avoid problems with the erase) and then delete
        :: it.

        if "%RETVAL%" EQU "YES" (

            call :ExecuteLoggedCommand attrib -s -h -r %1 /s /d
            call :ExecuteLoggedCommand attrib -s -h -r %1
            call :ExecuteLoggedCommand rmdir /s /q %1

        ) else (

            call :ExecuteLoggedCommand attrib -s -h -r %1
            call :ExecuteLoggedCommand erase /f /q %1
        )


    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::
:: %1 - Path of the key to delete (or simply the parent of single value to delete, named by %2).
:: %2 - Name of the value to delete (optional; if omitted, then deletes entire key named by %1).
::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:DeleteKeyOrValue

    setlocal


        :: If we have a second parameter (value name), then we attempt to delete it.  Otherwise,
        :: we attempt to the delete the entire key (which will include ALL values and subkeys!).

        if "%~2" NEQ "" (

            call :ExecuteLoggedCommand reg delete %1 /v %2 /f

        ) else (

            call :ExecuteLoggedCommand reg delete %1 /f
        )


    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::
:: %* - Any command line.
::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:ExecuteCommandAndEchoResult

    setlocal


        :: Execute the actual command, hiding any informational or diagnostic messages it might
        :: generate.  (The log will contain a general indication of success or failure for each
        :: command.)

        %* > nul 2>&1


        :: If we got a non-zero exit code, the command failed.  Otherwise, it worked!

        if ERRORLEVEL 1 (

            set Result=FAILED ^(exit code = %ERRORLEVEL%^)

        ) else (

            set Result=SUCCESS
        )


        :: Output the result.

        echo Command: %*
        echo Result:  %Result%


    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::
:: %* - Any command line.
::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:ExecuteLoggedCommand

    setlocal


        :: Just in case the command changes the current directory, let's save it "around" the
        :: command execution to ensure that we will still be in the same directory afterward.

        pushd .

        call :ExecuteCommandAndEchoResult %*            >> "%LogFile%"
        echo.                                           >> "%LogFile%"

        popd


    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:OpenLog

    setlocal

        echo Log opened: %DATE% %TIME%                  >> "%LogFile%"
        echo.                                           >> "%LogFile%"
        echo Present: %WasInstalled%                    >> "%LogFile%"
        echo Install: %InstallPath%                     >> "%LogFile%"
        echo Shared:  %SharedPath%                      >> "%LogFile%"
        echo Data:    %DataPath%                        >> "%LogFile%"
        echo Files:   %LocalRootPath%                   >> "%LogFile%"
        echo.                                           >> "%LogFile%"

    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:CloseLog

    setlocal

        echo Log closed: %DATE% %TIME%                  >> "%LogFile%"
        echo.                                           >> "%LogFile%"
        echo ------------------------------------------ >> "%LogFile%"
        echo.                                           >> "%LogFile%"

    endlocal & goto :EOF


::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::
:: %1 - Message to display to the user.
::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:PromptUser

    setlocal

        set /p Choice=%~1

    endlocal & set "RETVAL=%Choice%" & goto :EOF

